
package org.jhotdraw.util; 
import org.jhotdraw.framework.*; 
import org.jhotdraw.standard.StandardDrawing; 
import java.io.*; 
import java.util.*; 
import java.util.List; 
import java.awt.*; 
import javax.jdo.*; 
import javax.swing.*; 
import javax.swing.event.ListSelectionListener; 
import javax.swing.event.ListSelectionEvent; 
public  class  JDOStorageFormat  extends StandardStorageFormat {
		private Map pms;

		public JDOStorageFormat() {	super();	pms = CollectionsFactory.current().createMap();	Runtime.getRuntime().addShutdownHook(new Thread() {	public void run() {	Iterator iter = pms.values().iterator();	}	});	}

		protected String createFileExtension() {	return "j2";	}

		public String createFileDescription() {	return "Database (" + getFileExtension() + ")";	}

		public boolean isRestoreFormat() {	return true;	}

		public boolean isStoreFormat() {	return true;	}

		public String store(String fileName, Drawing storeDrawing) throws IOException {	PersistenceManager pm = getPersistenceManager(fileName);	String drawingName = null;	Drawing txnDrawing = crossTxnBoundaries(storeDrawing);	endTransaction(pm, false);	startTransaction(pm);	try {	Extent extent = pm.getExtent(StandardDrawing.class, true);	DrawingListModel listModel = new DrawingListModel(extent.iterator());	drawingName = showStoreDialog(listModel, storeDrawing);	if (drawingName != null) {	storeDrawing.setTitle(drawingName);	txnDrawing.setTitle(drawingName);	pm.makePersistent(txnDrawing);	}	}	finally {	endTransaction(pm, (drawingName != null));	}	startTransaction(pm);	return drawingName;	}

		public synchronized Drawing restore(String fileName) throws IOException {	PersistenceManager pm = getPersistenceManager(fileName);	endTransaction(pm, false);	startTransaction(pm);	Drawing restoredDrawing = null;	try {	Extent extent = pm.getExtent(StandardDrawing.class, true);	DrawingListModel listModel = new DrawingListModel(extent.iterator());	Drawing txnDrawing = showRestoreDialog(listModel);	if (txnDrawing != null) {	restoredDrawing = txnDrawing;	}	}	finally {	endTransaction(pm, false);	}	startTransaction(pm);	return restoredDrawing;	}

		private void retrieveAll(PersistenceManager pm, Figure figure) {	pm.retrieve(figure);	FigureEnumeration fe = figure.figures();	while (fe.hasNextFigure()) {	retrieveAll(pm, fe.nextFigure());	}	}

		private Drawing crossTxnBoundaries(Drawing originalDrawing) {	return (Drawing)((StandardDrawing)originalDrawing).clone();	}

		private synchronized PersistenceManager getPersistenceManager(String fileName) {	PersistenceManager pm = (PersistenceManager)pms.get(fileName);	if (pm == null) {	pm = createPersistenceManagerFactory(fileName).getPersistenceManager();	pms.put(fileName, pm);	}	return pm;	}

		private PersistenceManagerFactory createPersistenceManagerFactory(String dbFileName) {	Properties pmfProps = new Properties(); pmfProps.put( "javax.jdo.PersistenceManagerFactoryClass", "com.poet.jdo.PersistenceManagerFactories" ); pmfProps.put( "javax.jdo.option.ConnectionURL", "fastobjects://LOCAL/MyBase.j1" ); final PersistenceManagerFactory pmf = JDOHelper.getPersistenceManagerFactory( pmfProps );	return pmf;	}

		private static void startTransaction(PersistenceManager pm) {	if (!pm.currentTransaction().isActive()) {	pm.currentTransaction().begin();	}	}

		private static void endTransaction(PersistenceManager pm, boolean doCommit) {	if (pm.currentTransaction().isActive()) {	if (doCommit) {	pm.currentTransaction().commit();	}	else {	pm.currentTransaction().rollback();	}	}	}

		private String showStoreDialog(ListModel listModel, Drawing storeDrawing) {	final String msgString = "Specify a name for the drawing";	final JTextField nameTextField = new JTextField(storeDrawing.getTitle());	final JList dataList = new JList(listModel);	final JScrollPane dbContentScrollPane = new JScrollPane(dataList);	Object[] guiComponents = {msgString, dbContentScrollPane, nameTextField};	dataList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);	dataList.setValueIsAdjusting(true);	dataList.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) {	nameTextField.setText(dataList.getSelectedValue().toString());	}	});	final JOptionPane optionPane = new JOptionPane(	guiComponents,	JOptionPane.PLAIN_MESSAGE,	JOptionPane.OK_CANCEL_OPTION);	final JDialog dialog = optionPane.createDialog(null, "Restore a drawing from the database");	dialog.setVisible(true);	if ((optionPane.getValue() != null) && (optionPane.getValue().equals(new Integer(JOptionPane.OK_OPTION)))) {	return nameTextField.getText();	}	else {	return null;	}	}

		private Drawing showRestoreDialog(DrawingListModel listModel) {	final String msgString = "Select a drawing";	final JList dataList = new JList(listModel);	final JScrollPane dbContentScrollPane = new JScrollPane(dataList);	Object[] guiComponents = {msgString, dbContentScrollPane};	dataList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);	dataList.setValueIsAdjusting(true);	final JOptionPane optionPane = new JOptionPane(	guiComponents,	JOptionPane.PLAIN_MESSAGE,	JOptionPane.OK_CANCEL_OPTION);	final JDialog dialog = optionPane.createDialog(null, "Restore a drawing from the database");	dialog.setVisible(true);	if ((optionPane.getValue() != null)	&& (optionPane.getValue().equals(new Integer(JOptionPane.OK_OPTION)))	&& (dataList.getSelectedIndex() >= 0)	&& (dataList.getSelectedIndex() < dataList.getModel().getSize())) {	return listModel.getDrawingAt(dataList.getSelectedIndex());	}	else {	return null;	}	}

		static  class  DrawingListModel  extends AbstractListModel {
			private List myList;

			DrawingListModel(Iterator iter) {	myList = CollectionsFactory.current().createList();	while (iter.hasNext()) {	Object o = iter.next();	System.out.println("extent: " + o + " .. " + ((Drawing)o).getTitle());	myList.add(o);	}	}

			public Object getElementAt(int index) {	return getDrawingAt(index).getTitle();	}

			protected Drawing getDrawingAt(int index) {	return ((Drawing)myList.get(index));	}

			public int getSize() {	return myList.size();	}


	}

		static  class  DrawingSelector  extends JDialog {
			DrawingSelector() {	init();	}

			private void init() {	setTitle("Select Drawing");	getContentPane().setLayout(new BorderLayout());	getContentPane().add(new JLabel("Database content"), BorderLayout.NORTH);	setSize(200, 200);	}


	}

		public static void main(String[] args) {	DrawingSelector frame = new DrawingSelector();	try {	Drawing newDrawing = new StandardDrawing();	newDrawing.setTitle("TestDrawingName" + new Random(System.currentTimeMillis()).nextLong());	new JDOStorageFormat().store("base.j2", newDrawing);	System.exit(0);	}	catch (IOException e) {	e.printStackTrace();	}	}


}
